package org.jenkinsci.plugins.p4_client.workspace;

import hudson.Extension;
import hudson.model.AutoCompletionCandidates;
import hudson.util.FormValidation;

import java.util.List;
import java.util.regex.Pattern;

import org.jenkinsci.plugins.p4_client.client.CharsetTypes;
import org.jenkinsci.plugins.p4_client.client.ConnectionFactory;
import org.kohsuke.stapler.DataBoundConstructor;
import org.kohsuke.stapler.QueryParameter;

import com.perforce.p4java.client.IClient;
import com.perforce.p4java.client.IClientSummary;
import com.perforce.p4java.server.IOptionsServer;

public class TemplateWorkspaceImpl extends Workspace {

	private final String charset;
	private final String templateName;
	private final String prefix;

	public String getCharset() {
		return charset;
	}

	public String getTemplateName() {
		return templateName;
	}

	public String getPrefix() {
		return prefix;
	}

	@Override
	public WorkspaceType getType() {
		return WorkspaceType.TEMPLATED;
	}

	@DataBoundConstructor
	public TemplateWorkspaceImpl(String charset, String templateName,
			String prefix) {
		this.charset = charset;
		this.templateName = templateName;
		this.prefix = prefix;
	}

	@Extension
	public static final class DescriptorImpl extends WorkspaceDescriptor {

		@Override
		public String getDisplayName() {
			return "Template (view generated for each node)";
		}

		/**
		 * Provides auto-completion for workspace names. Stapler finds this
		 * method via the naming convention.
		 * 
		 * @param value
		 *            The text that the user entered.
		 */
		public AutoCompletionCandidates doAutoCompleteTemplateName(
				@QueryParameter String value) {

			AutoCompletionCandidates c = new AutoCompletionCandidates();
			try {
				IOptionsServer iserver = ConnectionFactory.getConnection();
				if (iserver != null && value.length() > 0) {
					String user = iserver.getUserName();
					List<IClientSummary> list;
					list = iserver.getClients(user, value + "*", 10);
					for (IClientSummary l : list) {
						c.add(l.getName());
					}
				}
			} catch (Exception e) {
			}

			return c;
		}

		public FormValidation doCheckTemplateName(@QueryParameter String value) {
			try {
				IOptionsServer p4 = ConnectionFactory.getConnection();
				IClient client = p4.getClient(value);
				if (client != null) {
					return FormValidation.ok();
				}
				return FormValidation.error("Unknown Client: " + value);
			} catch (Exception e) {
				return FormValidation.error("Perforce connection error.");
			}
		}

		public FormValidation doCheckPrefix(@QueryParameter final String value) {
			if (value == null)
				return FormValidation.error("Prefix path is mandatory.");

			Pattern regex = Pattern.compile(".*[-_\\.]$");
			if (regex.matcher(value).matches())
				return FormValidation.ok();

			return FormValidation.error("Prefix must end with '-', '_' or '.'");
		}
		
		/**
		 * Magic Stapler method; provides auto-completion for Charset types.
		 * 
		 * @param value
		 *            The text that the user entered.
		 */
		public AutoCompletionCandidates doAutoCompleteCharset(
				@QueryParameter String value) {
			AutoCompletionCandidates c = new AutoCompletionCandidates();
			for (String set : CharsetTypes.getCharsetsSince(20133)) {
				if (set.startsWith(value)) {
					c.add(set);
				}
			}
			return c;
		}
	}

	/**
	 * Builds Workspace name from prefix, node and project.
	 * 
	 * @return
	 */
	@Override
	public String getName() {
		StringBuffer sb = new StringBuffer();
		sb.append(prefix);
		sb.append((getNodeName() != "") ? getNodeName() : "master");
		sb.append("-");
		sb.append(getProjectName());
		return sb.toString();
	}
}
